unit Riemann_Zeta;

interface

function xRiemann_Zeta_Star_Function(s: extended): extended;

implementation

uses system.math, dirichlet_eta;

//------------------------------------------------------------------------------
// function xRiemann_Zeta_Star_Function(s : extended): extended;
//
//  Description:
//     This routine calculates the Riemannt zeta star function defined as
//                           zeta*(s) = zeta(s) - 1,
//     where zeta() is the Riemann zeta function for real s != 1.
//------------------------------------------------------------------------------
function xRiemann_Zeta_Star_Function(s: extended): extended;
var
  two_s: extended;
  eta: extended;
  temp: extended;
begin
  two_s := power(2.0, s - 1.0);
  eta := xDirichlet_Eta_Star_Function(s);
  if s = 1.0 then begin
    result := MAXDouble;
    exit;
  end;
  if s < 0.0  then begin
    two_s := 1.0 / two_s;
    result := (eta + two_s) / (1.0 - two_s);
    exit;
  end;
  temp := two_s - 1.0;
  result := (two_s * eta + 1.0) / temp;
end;

end.
