unit dirichlet_eta;

interface

  function xDirichlet_Eta_Function(s : extended): extended;
  function xDirichlet_Eta_Star_Function(s: extended): extended;

implementation

uses system.math, Gamma_Function, main;

//function xDirichlet_Eta_Star_Function(s: extended): extended; forward;
function Alternating_Series_Convergence_Acceleration(s: extended): extended;  forward;
function Sum_Reverse_Order(s : extended): extended; forward;
function Reflection_Coefficient(s: extended): extended; forward;

//-----------------------------------------------------------------------
// function Dirichlet_Eta_Functionis: extended): extended;
//
//     ̃[`́AEta֐vZ܂
//            etaisj=i1-2 ^i1-sjjzetaisjA
//     ŁAzetaij̓[}[[^֐łBs> 1̏ꍇA
//            etaisj= Sumi-1j^ik-1ji1 / k ^ sjA
//     ŁAv k = 1...etai1j= lni2j
//
// @@s <1̏ꍇAˎgp邱ƂɒӂĂ
//     s << 0̏ꍇ etaisǰX́A傫傫Aۓ
//@@͑傫ȐΌ덷쐬\܂B
//-----------------------------------------------------------------------
function xDirichlet_Eta_Function(s : extended): extended;
begin
  if (s >= 64.0) then begin
    result :=  1.0;
    exit;
  end;
  result := 1.0 + xDirichlet_Eta_Star_Function(s);
end;

//--------------------------------------------------------------------------------
// function xDirichlet_Eta_Star_Functionis: extended): extended;
//    ̃[`́Â悤ɒ`ꂽDirichlet's eta֐vZ܂
//              eta *isj= etaisj-1A
//    ŁAetaij̓fBÑC[^֐łB//
//--------------------------------------------------------------------------------
function xDirichlet_Eta_Star_Function(s: extended): extended;
var
  reflection_coefficientd: extended;
begin
  if s >= 0.0 then  begin
    if  s < 18.0 then
      result := Alternating_Series_Convergence_Acceleration(s)
    else
      result := Sum_Reverse_Order(s);
    exit;
  end;

  // Use the reflection formula
  reflection_coefficientd := Reflection_Coefficient(s);
  result := reflection_coefficientd * xDirichlet_Eta_Star_Function(1.0 - s)
          + (reflection_coefficientd - 1.0);
end;

//---------------------------------------------------------------------------
// function Reflection_Coefficientis: extended): extemded;
//    fBÑC[^֐̔ˌ͎̂ƂłB
//    etaisj= {[2 *i1-2 ^i1-sjj/i1-2 ^ sj] *Gamma(1-s)
//   i2 * pij^i1-sj* cosii1-sj* pi / 2j} * etai1-sj
//    ŁAs <0͕̎łB
//
//    ̃[`͌WԂ܂
//     [2 *i1-2^(1-s))/(1-2^s)]Gamma(1-s)/(2 * pi)^(1-s)*cos((1-s)* pi/2j
//    ŁAs <0͕̎łB
//---------------------------------------------------------------------------
function Reflection_Coefficient(s: extended): extended;
var
  one_s: extended;
  k : integer;
  v : extended;
  c : extended;
  x, temp : extended;
begin
  one_s := 1.0 - s;
  k := trunc(one_s / 4.0);
  v := one_s - 4.0 * k;
  c := cos(v * pi / 2);
  if abs(c) < 1.8 * LDBL_EPSILON then begin
    result := 0.0;
    exit;
  end;
  temp := power(2.0, one_s);
  x := (1.0 - temp) / (temp - 2.0);
  x := x + x;
  x := x * c;
  x := x * xGamma_Function(one_s);
  x := x / power(pi, one_s);
  result := x;
end;

//-----------------------------------------------------------------------------
// function Alternating_Series_Convergence_Acceleration(s: extended): extended;
//    ̃[`́As <= 18Eta֐vZ܂B
//    Ǝ܂B
//     Technique:
//     Given an alternating series: Sum (-1)^k a[k] summmed over k = 0,...
//     where a[k] > 0 for all k.  If there exits a positive function w(x)
//     defined on [0,1] such that a[k] = I[0,1] x^k w(x) dx (the integral
//     from 0 to 1 with respect to x of x^k w(x), then
//          Sum (-1)^k a[k] = Sum (-1)^k I[0,1] x^k w(x) dx
//                          = I[0,1] Sum (-1)^k x^k w(x) dx
//                          = I[0,1] w(x) / (1 + x) dx
//
//     Let S = I[0,1] w(x) / (1 + x) dx.
//     Given a polynomial P(x) of degree n such that P(-1) != 0,
//          P(x) = p[0] - p[1] x + p[2] x^2 - ... (-1)^n p[n] x^n,
//     define S[n] = I[0,1] (P(-1) - P(x)) w(x) / (1+x) dx / P(-1).
//     Then   S[n] = S - I[0,1] P(x) w(x) / (1+x) dx / P(-1)
//     so,   |S[n] - S| <= I[0,1] |P(x)| w(x) / (1+x) dx / |P(x)|
//           |S[n] - S| <= (M / |P(-1)|) S, where M = max |P(x)|, x in [0,1].
//
//     After a little algebra,
//               S[n] = (1/P(-1)) Sum Sum p[j] [(-1)^k a[k]],
//     where the first sum is over k = 0,...,n-1 and the second sum is over
//     j = k+1,...,n.
//     Let d[k] = Sum p[j] summed for j = k+1,...,n for  k = n-1, ..., 0, then
//        S[n] = (1/c[0]) Sum d[k] (-1)^k a[k] summed k = 0,...,n-1.
//
//     The choice for P(x) programmed here is (-1)^n * T*[n](x), where T*[n]
//     is the shifted Chebyshev polynomial of the first kind of degree n.
//     With this choice of P(x),
//              P(x) = Sum {( n / (n+j) ) C(n+j,2j) 4^j (-x)^j},
//     where the sum is over j = 0,...,n and C(n+j,2j) = (n+j)!/[(2j)!(n-j)!]
//     and M = 1, P(-1) >= (1/2) (3 + sqrt(8))^n.
//------------------------------------------------------------------------------
function Alternating_Series_Convergence_Acceleration(s: extended): extended;
const
  d: array[0..28] of extended = (
     1.362725501650887306817e+21, 1.362725501650887306816e+21,
     1.362725501650887305248e+21, 1.362725501650886896000e+21,
     1.362725501650844334208e+21, 1.362725501648488235008e+21,
     1.362725501568066715648e+21, 1.362725499718371770368e+21,
     1.362725469310199922688e+21, 1.362725096810094788608e+21,
     1.362721590926752350208e+21, 1.362695647390018306048e+21,
     1.362542007743905005568e+21, 1.361803869444099801088e+21,
     1.358896740140251611136e+21, 1.349437033675348770816e+21,
     1.323863206542645919744e+21, 1.266218975223368122368e+21,
     1.157712186857668739072e+21, 9.872015194258554224640e+20,
     7.640581139674368573440e+20, 5.220333434317674905600e+20,
     3.061506212814840135680e+20, 1.496014168469232680960e+20,
     5.884825485587356057600e+19, 1.781624012587768217600e+19,
     3.882102878793367552000e+18, 5.404319552844595200000e+17,
     3.602879701896396800000e+16);
var
  term: array[0..27] of extended;
  sum: extended;
  k: integer;
begin
  k := 0;
  while k <= 27 do  begin
    term[k] := d[k + 1] * power(k + 2,-s);
    inc(k);
    term[k] := -d[k + 1] * power(k + 2,-s);
    inc(k)
  end;
  sum := term[27];
  for k := 26 downto 0 do sum := sum + term[k];
  sum := sum / d[0];

  result := -sum;
end;

//-------------------------------------------------------------------------
// long double Sum_Reverse_Orderilong double sj//
//
//  ̃[`́As> = 18Eta֐vZ܂ipower()ւ̌Ăяo28j
//         etaisj= Sumi-1j^ik-1ji1 / k ^ sjA
//   ŁAvk = 1A...ōv܂
//
//     Since the series is alternating, the absolute error is less than the
//     absolute value of the first neglected term.  The number of terms which
//     are summed is determined by calculating successive lower and upper
//     bounds until the two bounds are equal (within truncation errors).
//     The final result is then summed starting with the last term and
//     terminating with the first term, (this is slightly more accurate).
//--------------------------------------------------------------------------
function Sum_Reverse_Order(s : extended): extended;
var
  term: array[0..29] of extended;
  lower_bound: extended;
  upper_bound: extended;
  sum: extended;
  k: integer;
begin
  lower_bound := -power(2.0, -s);
  term[0] := lower_bound;
  term[1] := power(3.0 ,-s);
  upper_bound := term[1] + lower_bound;
  if lower_bound = upper_bound then begin
    result := upper_bound;
    exit;
  end;
  k := 4;
  while k <= 31 do  begin
    term[k-2] := -power(k, -s);
    lower_bound := upper_bound + term[k - 2];
    if lower_bound = upper_bound then break;
    inc(k);
    term[k - 2] := power(k, -s);
    upper_bound := lower_bound + term[ k- 2];
    if lower_bound = upper_bound then break;
    inc(k);
  end;
  k := k - 2;
  sum := term[k];
  for k := k - 1 downto 0 do  sum := sum + term[k];
  result := sum;
end;

end.
